(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.OwnEasing = factory());
}(this, (function () { 'use strict';

'use strict';
var _slicedToArray = (function () { function sliceIterator (arr, i) { var _arr = []; var _n = true; var _d = false; var _e; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i['return']) _i['return'](); } finally { if (_d) throw _e } } return _arr } return function (arr, i) { if (Array.isArray(arr)) { return arr } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i) } else { throw new TypeError('Invalid attempt to destructure non-iterable instance') } } }());

// I extracted this from the a2c function from
// SVG path – https://github.com/fontello/svgpath
//
// All credit goes to:
//
// Sergey Batishchev – https://github.com/snb2013
// Vitaly Puzrin – https://github.com/puzrin
// Alex Kocharin – https://github.com/rlidwka

var TAU = Math.PI * 2;

var mapToEllipse = function mapToEllipse (_ref, rx, ry, cosphi, sinphi, centerx, centery) {
  var x = _ref.x,
    y = _ref.y;

  x *= rx;
  y *= ry;

  var xp = cosphi * x - sinphi * y;
  var yp = sinphi * x + cosphi * y;

  return {
    x: xp + centerx,
    y: yp + centery
  }
};

var approxUnitArc = function approxUnitArc (ang1, ang2) {
  var a = 4 / 3 * Math.tan(ang2 / 4);

  var x1 = Math.cos(ang1);
  var y1 = Math.sin(ang1);
  var x2 = Math.cos(ang1 + ang2);
  var y2 = Math.sin(ang1 + ang2);

  return [{
    x: x1 - y1 * a,
    y: y1 + x1 * a
  }, {
    x: x2 + y2 * a,
    y: y2 - x2 * a
  }, {
    x: x2,
    y: y2
  }]
};

var vectorAngle = function vectorAngle (ux, uy, vx, vy) {
  var sign = ux * vy - uy * vx < 0 ? -1 : 1;
  var umag = Math.sqrt(ux * ux + uy * uy);
  var vmag = Math.sqrt(ux * ux + uy * uy);
  var dot = ux * vx + uy * vy;

  var div = dot / (umag * vmag);

  if (div > 1) {
    div = 1;
  }

  if (div < -1) {
    div = -1;
  }

  return sign * Math.acos(div)
};

var getArcCenter = function getArcCenter (px, py, cx, cy, rx, ry, largeArcFlag, sweepFlag, sinphi, cosphi, pxp, pyp) {
  var rxsq = Math.pow(rx, 2);
  var rysq = Math.pow(ry, 2);
  var pxpsq = Math.pow(pxp, 2);
  var pypsq = Math.pow(pyp, 2);

  var radicant = rxsq * rysq - rxsq * pypsq - rysq * pxpsq;

  if (radicant < 0) {
    radicant = 0;
  }

  radicant /= rxsq * pypsq + rysq * pxpsq;
  radicant = Math.sqrt(radicant) * (largeArcFlag === sweepFlag ? -1 : 1);

  var centerxp = radicant * rx / ry * pyp;
  var centeryp = radicant * -ry / rx * pxp;

  var centerx = cosphi * centerxp - sinphi * centeryp + (px + cx) / 2;
  var centery = sinphi * centerxp + cosphi * centeryp + (py + cy) / 2;

  var vx1 = (pxp - centerxp) / rx;
  var vy1 = (pyp - centeryp) / ry;
  var vx2 = (-pxp - centerxp) / rx;
  var vy2 = (-pyp - centeryp) / ry;

  var ang1 = vectorAngle(1, 0, vx1, vy1);
  var ang2 = vectorAngle(vx1, vy1, vx2, vy2);

  if (sweepFlag === 0 && ang2 > 0) {
    ang2 -= TAU;
  }

  if (sweepFlag === 1 && ang2 < 0) {
    ang2 += TAU;
  }

  return [centerx, centery, ang1, ang2]
};

var arcToBezier = function arcToBezier (_ref2) {
  var px = _ref2.px,
    py = _ref2.py,
    cx = _ref2.cx,
    cy = _ref2.cy,
    rx = _ref2.rx,
    ry = _ref2.ry,
    _ref2$xAxisRotation = _ref2.xAxisRotation,
    xAxisRotation = _ref2$xAxisRotation === undefined ? 0 : _ref2$xAxisRotation,
    _ref2$largeArcFlag = _ref2.largeArcFlag,
    largeArcFlag = _ref2$largeArcFlag === undefined ? 0 : _ref2$largeArcFlag,
    _ref2$sweepFlag = _ref2.sweepFlag,
    sweepFlag = _ref2$sweepFlag === undefined ? 0 : _ref2$sweepFlag;

  var curves = [];

  if (rx === 0 || ry === 0) {
    return []
  }

  var sinphi = Math.sin(xAxisRotation * TAU / 360);
  var cosphi = Math.cos(xAxisRotation * TAU / 360);

  var pxp = cosphi * (px - cx) / 2 + sinphi * (py - cy) / 2;
  var pyp = -sinphi * (px - cx) / 2 + cosphi * (py - cy) / 2;

  if (pxp === 0 && pyp === 0) {
    return []
  }

  rx = Math.abs(rx);
  ry = Math.abs(ry);

  var lambda = Math.pow(pxp, 2) / Math.pow(rx, 2) + Math.pow(pyp, 2) / Math.pow(ry, 2);

  if (lambda > 1) {
    rx *= Math.sqrt(lambda);
    ry *= Math.sqrt(lambda);
  }

  var _getArcCenter = getArcCenter(px, py, cx, cy, rx, ry, largeArcFlag, sweepFlag, sinphi, cosphi, pxp, pyp),
    _getArcCenter2 = _slicedToArray(_getArcCenter, 4),
    centerx = _getArcCenter2[0],
    centery = _getArcCenter2[1],
    ang1 = _getArcCenter2[2],
    ang2 = _getArcCenter2[3];

  var segments = Math.max(Math.ceil(Math.abs(ang2) / (TAU / 4)), 1);

  ang2 /= segments;

  for (var i = 0; i < segments; i++) {
    curves.push(approxUnitArc(ang1, ang2));
    ang1 += ang2;
  }

  return curves.map(function (curve) {
    var _mapToEllipse = mapToEllipse(curve[0], rx, ry, cosphi, sinphi, centerx, centery),
      x1 = _mapToEllipse.x,
      y1 = _mapToEllipse.y;

    var _mapToEllipse2 = mapToEllipse(curve[1], rx, ry, cosphi, sinphi, centerx, centery),
      x2 = _mapToEllipse2.x,
      y2 = _mapToEllipse2.y;

    var _mapToEllipse3 = mapToEllipse(curve[2], rx, ry, cosphi, sinphi, centerx, centery),
      x = _mapToEllipse3.x,
      y = _mapToEllipse3.y;

    return { x1: x1, y1: y1, x2: x2, y2: y2, x: x, y: y }
  })
};

var toPoints = function toPoints (props) {

  switch (props.type) {
    case 'circle':
      return getPointsFromCircle(props)
    case 'ellipse':
      return getPointsFromEllipse(props)
    case 'line':
      return getPointsFromLine(props)
    case 'path':
      return getPointsFromPath(props)
    case 'polygon':
      return getPointsFromPolygon(props)
    case 'polyline':
      return getPointsFromPolyline(props)
    case 'rect':
      return getPointsFromRect(props)
    case 'g':
      return getPointsFromG(props)
    default:
      throw new Error('Not a valid shape type')
  }
};

var getPointsFromCircle = function getPointsFromCircle (_ref2) {
  var cx = _ref2.cx,
    cy = _ref2.cy,
    r = _ref2.r;

  return [{ x: cx, y: cy - r, moveTo: true }, { x: cx, y: cy + r, curve: { type: 'arc', rx: r, ry: r, sweepFlag: 1 } }, { x: cx, y: cy - r, curve: { type: 'arc', rx: r, ry: r, sweepFlag: 1 } }]
};

var getPointsFromEllipse = function getPointsFromEllipse (_ref3) {
  var cx = _ref3.cx,
    cy = _ref3.cy,
    rx = _ref3.rx,
    ry = _ref3.ry;

  return [{ x: cx, y: cy - ry, moveTo: true }, { x: cx, y: cy + ry, curve: { type: 'arc', rx: rx, ry: ry, sweepFlag: 1 } }, { x: cx, y: cy - ry, curve: { type: 'arc', rx: rx, ry: ry, sweepFlag: 1 } }]
};

var getPointsFromLine = function getPointsFromLine (_ref4) {
  var x1 = _ref4.x1,
    x2 = _ref4.x2,
    y1 = _ref4.y1,
    y2 = _ref4.y2;

  return [{ x: x1, y: y1, moveTo: true }, { x: x2, y: y2 }]
};

var validCommands = /[MmLlHhVvCcSsQqTtAaZz]/g;

var commandLengths = {
  A: 7,
  C: 6,
  H: 1,
  L: 2,
  M: 2,
  Q: 4,
  S: 4,
  T: 2,
  V: 1,
  Z: 0
};

var relativeCommands = ['a', 'c', 'h', 'l', 'm', 'q', 's', 't', 'v'];

var isRelative = function isRelative (command) {
  return relativeCommands.indexOf(command) !== -1
};

var optionalArcKeys = ['xAxisRotation', 'largeArcFlag', 'sweepFlag'];

var getCommands = function getCommands (d) {
  return d.match(validCommands)
};

var getParams = function getParams (d) {
  return d.split(validCommands).map(function (v) {
    return v.replace(/[0-9]+-/g, function (m) {
      return m.slice(0, -1) + ' -'
    })
  }).map(function (v) {
    return v.replace(/\.[0-9]+/g, function (m) {
      return m + ' '
    })
  }).map(function (v) {
    return v.trim()
  }).filter(function (v) {
    return v.length > 0
  }).map(function (v) {
    return v.split(/[ ,]+/).map(parseFloat).filter(function (n) {
      return !isNaN(n)
    })
  })
};

var getPointsFromPath = function getPointsFromPath (_ref5) {
  var d = _ref5.d;

  var commands = getCommands(d);
  var params = getParams(d);

  var points = [];

  var moveTo = void 0;

  for (var i = 0, l = commands.length; i < l; i++) {
    var command = commands[i];
    var upperCaseCommand = command.toUpperCase();
    var commandLength = commandLengths[upperCaseCommand];
    var relative = isRelative(command);
    var prevPoint = i === 0 ? null : points[points.length - 1];

    if (commandLength > 0) {
      var commandParams = params.shift();
      var iterations = commandParams.length / commandLength;

      for (var j = 0; j < iterations; j++) {
        switch (upperCaseCommand) {
          case 'M':
            var x = (relative && prevPoint ? prevPoint.x : 0) + commandParams.shift();
            var y = (relative && prevPoint ? prevPoint.y : 0) + commandParams.shift();

            moveTo = { x: x, y: y };

            points.push({ x: x, y: y, moveTo: true });

            break

          case 'L':
            points.push({
              x: (relative ? prevPoint.x : 0) + commandParams.shift(),
              y: (relative ? prevPoint.y : 0) + commandParams.shift()
            });

            break

          case 'H':
            points.push({
              x: (relative ? prevPoint.x : 0) + commandParams.shift(),
              y: prevPoint.y
            });

            break

          case 'V':
            points.push({
              x: prevPoint.x,
              y: (relative ? prevPoint.y : 0) + commandParams.shift()
            });

            break

          case 'A':
            points.push({
              curve: {
                type: 'arc',
                rx: commandParams.shift(),
                ry: commandParams.shift(),
                xAxisRotation: commandParams.shift(),
                largeArcFlag: commandParams.shift(),
                sweepFlag: commandParams.shift()
              },
              x: (relative ? prevPoint.x : 0) + commandParams.shift(),
              y: (relative ? prevPoint.y : 0) + commandParams.shift()
            });

            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError;

            try {
              for (var _iterator = optionalArcKeys[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var k = _step.value;

                if (points[points.length - 1]['curve'][k] === 0) {
                  delete points[points.length - 1]['curve'][k];
                }
              }
            } catch (err) {
              _didIteratorError = true;
              _iteratorError = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }
              } finally {
                if (_didIteratorError) {
                  throw _iteratorError
                }
              }
            }

            break

          case 'C':
            points.push({
              curve: {
                type: 'cubic',
                x1: (relative ? prevPoint.x : 0) + commandParams.shift(),
                y1: (relative ? prevPoint.y : 0) + commandParams.shift(),
                x2: (relative ? prevPoint.x : 0) + commandParams.shift(),
                y2: (relative ? prevPoint.y : 0) + commandParams.shift()
              },
              x: (relative ? prevPoint.x : 0) + commandParams.shift(),
              y: (relative ? prevPoint.y : 0) + commandParams.shift()
            });

            break

          case 'S':
            var sx2 = (relative ? prevPoint.x : 0) + commandParams.shift();
            var sy2 = (relative ? prevPoint.y : 0) + commandParams.shift();
            var sx = (relative ? prevPoint.x : 0) + commandParams.shift();
            var sy = (relative ? prevPoint.y : 0) + commandParams.shift();

            var diff = {};

            var sx1 = void 0;
            var sy1 = void 0;

            if (prevPoint.curve && prevPoint.curve.type === 'cubic') {
              diff.x = Math.abs(prevPoint.x - prevPoint.curve.x2);
              diff.y = Math.abs(prevPoint.y - prevPoint.curve.y2);
              sx1 = prevPoint.x < prevPoint.curve.x2 ? prevPoint.x - diff.x : prevPoint.x + diff.x;
              sy1 = prevPoint.y < prevPoint.curve.y2 ? prevPoint.y - diff.y : prevPoint.y + diff.y;
            } else {
              diff.x = Math.abs(sx - sx2);
              diff.y = Math.abs(sy - sy2);
              sx1 = prevPoint.x;
              sy1 = prevPoint.y;
            }

            points.push({ curve: { type: 'cubic', x1: sx1, y1: sy1, x2: sx2, y2: sy2 }, x: sx, y: sy });

            break

          case 'Q':
            points.push({
              curve: {
                type: 'quadratic',
                x1: (relative ? prevPoint.x : 0) + commandParams.shift(),
                y1: (relative ? prevPoint.y : 0) + commandParams.shift()
              },
              x: (relative ? prevPoint.x : 0) + commandParams.shift(),
              y: (relative ? prevPoint.y : 0) + commandParams.shift()
            });

            break

          case 'T':
            var tx = (relative ? prevPoint.x : 0) + commandParams.shift();
            var ty = (relative ? prevPoint.y : 0) + commandParams.shift();

            var tx1 = void 0;
            var ty1 = void 0;

            if (prevPoint.curve && prevPoint.curve.type === 'quadratic') {
              var _diff = {
                x: Math.abs(prevPoint.x - prevPoint.curve.x1),
                y: Math.abs(prevPoint.y - prevPoint.curve.y1)
              };

              tx1 = prevPoint.x < prevPoint.curve.x1 ? prevPoint.x - _diff.x : prevPoint.x + _diff.x;
              ty1 = prevPoint.y < prevPoint.curve.y1 ? prevPoint.y - _diff.y : prevPoint.y + _diff.y;
            } else {
              tx1 = prevPoint.x;
              ty1 = prevPoint.y;
            }

            points.push({ curve: { type: 'quadratic', x1: tx1, y1: ty1 }, x: tx, y: ty });

            break
        }
      }
    } else {
      if (prevPoint.x !== moveTo.x || prevPoint.y !== moveTo.y) {
        points.push({ x: moveTo.x, y: moveTo.y });
      }
    }
  }

  return points
};

var getPointsFromPolygon = function getPointsFromPolygon (_ref6) {
  var points = _ref6.points;

  return getPointsFromPoints({ closed: true, points: points })
};

var getPointsFromPolyline = function getPointsFromPolyline (_ref7) {
  var points = _ref7.points;

  return getPointsFromPoints({ closed: false, points: points })
};

var getPointsFromPoints = function getPointsFromPoints (_ref8) {
  var closed = _ref8.closed,
    points = _ref8.points;

  var numbers = points.split(/[\s,]+/).map(function (n) {
    return parseFloat(n)
  });

  var p = numbers.reduce(function (arr, point, i) {
    if (i % 2 === 0) {
      arr.push({ x: point });
    } else {
      arr[(i - 1) / 2].y = point;
    }

    return arr
  }, []);

  if (closed) {
	var firstPoint = p[0];
    p.push({x:firstPoint.x,y:firstPoint.y});
  }

  p[0].moveTo = true;

  return p
};

var getPointsFromRect = function getPointsFromRect (_ref9) {
  var height = _ref9.height,
    rx = _ref9.rx,
    ry = _ref9.ry,
    width = _ref9.width,
    x = _ref9.x,
    y = _ref9.y;

  if (rx || ry) {
    return getPointsFromRectWithCornerRadius({
      height: height,
      rx: rx || ry,
      ry: ry || rx,
      width: width,
      x: x,
      y: y
    })
  }

  return getPointsFromBasicRect({ height: height, width: width, x: x, y: y })
};

var getPointsFromBasicRect = function getPointsFromBasicRect (_ref10) {
  var height = _ref10.height,
    width = _ref10.width,
    x = _ref10.x,
    y = _ref10.y;

  return [{ x: x, y: y, moveTo: true }, { x: x + width, y: y }, { x: x + width, y: y + height }, { x: x, y: y + height }, { x: x, y: y }]
};

var getPointsFromRectWithCornerRadius = function getPointsFromRectWithCornerRadius (_ref11) {
  var height = _ref11.height,
    rx = _ref11.rx,
    ry = _ref11.ry,
    width = _ref11.width,
    x = _ref11.x,
    y = _ref11.y;

  var curve = { type: 'arc', rx: rx, ry: ry, sweepFlag: 1 };

  return [{ x: x + rx, y: y, moveTo: true }, { x: x + width - rx, y: y }, { x: x + width, y: y + ry, curve: curve }, { x: x + width, y: y + height - ry }, { x: x + width - rx, y: y + height, curve: curve }, { x: x + rx, y: y + height }, { x: x, y: y + height - ry, curve: curve }, { x: x, y: y + ry }, { x: x + rx, y: y, curve: curve }]
};

var getPointsFromG = function getPointsFromG (_ref12) {
  var shapes = _ref12.shapes;
  return shapes.map(function (s) {
    return toPoints(s)
  })
};

// AntiGrain approximate bezier curve algorithm
// ported by @mattdesl
// thank you
// you're amazing

var vec2 = function (x, y) {
	return {
		x: x,
		y: y
	}
};

var recursionLimit = 20;
	var fltEpsilon = 1.19209290e-7;
	var pathDistEpsilon = 1.0;

	var curveAngleToleranceEpsilon = 0.01;
	var mAngleTolerance = 0;
	var mCuspLimit = 0;

	/// /// Based on:
	/// /// https://github.com/pelson/antigrain/blob/master/agg-2.4/src/agg_curves.cpp

function CubicBezierApprox(x1, y1, x2, y2, x3, y3, x4, y4, points, distanceTolerance) {
	if (!points) {
		points = [];
	}
	points.push(vec2(x1, y1));
	cubicApproxRecursive(x1, y1, x2, y2, x3, y3, x4, y4, points, distanceTolerance || pathDistEpsilon, 0);
	points.push(vec2(x4, y4));
	return points
}

function QuadraticBezierApprox(x1, y1, x2, y2, x3, y3, points, distanceTolerance) {
	if (!points) {
		points = [];
	}
	points.push(vec2(x1, y1));
	quadraticApproxRecursive(x1, y1, x2, y2, x3, y3, points, distanceTolerance || pathDistEpsilon, 0);
	points.push(vec2(x4, y4));
	return points
}

function cubicApproxRecursive(x1, y1, x2, y2, x3, y3, x4, y4, points, distanceTolerance, level) {
	if (level > recursionLimit) {
		return
	}

	var pi = Math.PI;

		// Calculate all the mid-points of the line segments
		// ----------------------
		var x12 = (x1 + x2) / 2;
		var y12 = (y1 + y2) / 2;
		var x23 = (x2 + x3) / 2;
		var y23 = (y2 + y3) / 2;
		var x34 = (x3 + x4) / 2;
		var y34 = (y3 + y4) / 2;
		var x123 = (x12 + x23) / 2;
		var y123 = (y12 + y23) / 2;
		var x234 = (x23 + x34) / 2;
		var y234 = (y23 + y34) / 2;
		var x1234 = (x123 + x234) / 2;
		var y1234 = (y123 + y234) / 2;

		if (level > 0) { // Enforce subdivision first time
			// Try to approximate the full cubic curve by a single straight line
			// ------------------
			var dx = x4 - x1;
				var dy = y4 - y1;

				var d2 = Math.abs((x2 - x4) * dy - (y2 - y4) * dx);
				var d3 = Math.abs((x3 - x4) * dy - (y3 - y4) * dx);

				var da1,
			da2;

			if (d2 > fltEpsilon && d3 > fltEpsilon) {
				// Regular care
				// -----------------
				if ((d2 + d3) * (d2 + d3) <= distanceTolerance * (dx * dx + dy * dy)) {
					// If the curvature doesn't exceed the distanceTolerance value
					// we tend to finish subdivisions.
					// ----------------------
					if (mAngleTolerance < curveAngleToleranceEpsilon) {
						points.push(vec2(x1234, y1234));
						return
					}

					// Angle & Cusp Condition
					// ----------------------
					var a23 = Math.atan2(y3 - y2, x3 - x2);
						da1 = Math.abs(a23 - Math.atan2(y2 - y1, x2 - x1));
						da2 = Math.abs(Math.atan2(y4 - y3, x4 - x3) - a23);
						if (da1 >= pi) {
							da1 = 2 * pi - da1;
						}
						if (da2 >= pi) {
							da2 = 2 * pi - da2;
						}

						if (da1 + da2 < mAngleTolerance) {
							// Finally we can stop the recursion
							// ----------------------
							points.push(vec2(x1234, y1234));
							return
						}

						if (mCuspLimit !== 0.0) {
							if (da1 > mCuspLimit) {
								points.push(vec2(x2, y2));
								return
							}

							if (da2 > mCuspLimit) {
								points.push(vec2(x3, y3));
								return
							}
						}
				}
			} else {
				if (d2 > fltEpsilon) {
					// p1,p3,p4 are collinear, p2 is considerable
					// ----------------------
					if (d2 * d2 <= distanceTolerance * (dx * dx + dy * dy)) {
						if (mAngleTolerance < curveAngleToleranceEpsilon) {
							points.push(vec2(x1234, y1234));
							return
						}

						// Angle Condition
						// ----------------------
						da1 = Math.abs(Math.atan2(y3 - y2, x3 - x2) - Math.atan2(y2 - y1, x2 - x1));
							if (da1 >= pi) {
								da1 = 2 * pi - da1;
							}

							if (da1 < mAngleTolerance) {
								points.push(vec2(x2, y2));
								points.push(vec2(x3, y3));
								return
							}

							if (mCuspLimit !== 0.0) {
								if (da1 > mCuspLimit) {
									points.push(vec2(x2, y2));
									return
								}
							}
					}
				} else if (d3 > fltEpsilon) {
					// p1,p2,p4 are collinear, p3 is considerable
					// ----------------------
					if (d3 * d3 <= distanceTolerance * (dx * dx + dy * dy)) {
						if (mAngleTolerance < curveAngleToleranceEpsilon) {
							points.push(vec2(x1234, y1234));
							return
						}

						// Angle Condition
						// ----------------------
						da1 = Math.abs(Math.atan2(y4 - y3, x4 - x3) - Math.atan2(y3 - y2, x3 - x2));
							if (da1 >= pi) {
								da1 = 2 * pi - da1;
							}

							if (da1 < mAngleTolerance) {
								points.push(vec2(x2, y2));
								points.push(vec2(x3, y3));
								return
							}

							if (mCuspLimit !== 0.0) {
								if (da1 > mCuspLimit) {
									points.push(vec2(x3, y3));
									return
								}
							}
					}
				} else {
					// Collinear case
					// -----------------
					dx = x1234 - (x1 + x4) / 2;
						dy = y1234 - (y1 + y4) / 2;
						if (dx * dx + dy * dy <= distanceTolerance) {
							points.push(vec2(x1234, y1234));
							return
						}
				}
			}
		}

		// Continue subdivision
		// ----------------------
		cubicApproxRecursive(x1, y1, x12, y12, x123, y123, x1234, y1234, points, distanceTolerance, level + 1);
		cubicApproxRecursive(x1234, y1234, x234, y234, x34, y34, x4, y4, points, distanceTolerance, level + 1);
}

function quadraticApproxRecursive(x1, y1, x2, y2, x3, y3, points, distanceTolerance, level) {
	if (level > RECURSION_LIMIT) {
		return
	}

	var pi = Math.PI;

		// Calculate all the mid-points of the line segments
		// ----------------------
		var x12 = (x1 + x2) / 2;
		var y12 = (y1 + y2) / 2;
		var x23 = (x2 + x3) / 2;
		var y23 = (y2 + y3) / 2;
		var x123 = (x12 + x23) / 2;
		var y123 = (y12 + y23) / 2;

		var dx = x3 - x1;
		var dy = y3 - y1;
		var d = Math.abs((x2 - x3) * dy - (y2 - y3) * dx);

		if (d > FLT_EPSILON) {
			// Regular care
			// -----------------
			if (d * d <= distanceTolerance * (dx * dx + dy * dy)) {
				// If the curvature doesn't exceed the distance_tolerance value
				// we tend to finish subdivisions.
				// ----------------------
				if (m_angle_tolerance < curve_angle_tolerance_epsilon) {
					points.push(vec2(x123, y123));
					return
				}

				// Angle & Cusp Condition
				// ----------------------
				var da = Math.abs(Math.atan2(y3 - y2, x3 - x2) - Math.atan2(y2 - y1, x2 - x1));
					if (da >= pi) {
						da = 2 * pi - da;
					}

					if (da < m_angle_tolerance) {
						// Finally we can stop the recursion
						// ----------------------
						points.push(vec2(x123, y123));
						return
					}
			}
		} else {
			// Collinear case
			// -----------------
			dx = x123 - (x1 + x3) / 2;
				dy = y123 - (y1 + y3) / 2;
				if (dx * dx + dy * dy <= distanceTolerance) {
					points.push(vec2(x123, y123));
					return
				}
		}

		// Continue subdivision
		// ----------------------
		quadraticApproxRecursive(x1, y1, x12, y12, x123, y123, points, distanceTolerance, level + 1);
		quadraticApproxRecursive(x123, y123, x23, y23, x3, y3, points, distanceTolerance, level + 1);
}

// Mod by @dalisoft


function ApproximateCurves(points) {
	if (typeof points === 'string') {
		points = toPoints({
				type: "path",
				d: points
			});
	} else if (typeof points === 'object' && points.type) {
		points = toPoints(points);
	}
	var i = 0,
	p,
	p0,
	type,
	x1,
	y1,
	x2,
	y2,
	_straightLines;
	while (i < points.length) {
		p = points[i];
			if (p.curve) {
				p0 = points[i - 1];
					type = p.curve.type;
					x1 = p.curve.x1;
					y1 = p.curve.y1;
					x2 = p.curve.x2;
					y2 = p.curve.y2;
					if (type === 'arc') {
						var curves = arcToBezier({
								px: p0.x,
								py: p0.y,
								cx: p.x,
								cy: p.y,
								rx: p.curve.rx,
								ry: p.curve.ry,
								xAxisRotation: p.curve.xAxisRotation,
								largeArcFlag: p.curve.largeArcFlag,
								sweepFlag: p.curve.sweepFlag
							});

							points.splice(i, 1);
							for (var i2 = 0, len = curves.length; i2 < len; i2++) {
								var _ref = curves[i],
								x1 = _ref.x1,
								y1 = _ref.y1,
								x2 = _ref.x2,
								y2 = _ref.y2,
								x = _ref.x,
								y = _ref.y;

									points.splice(i + i2, 0, {
										x: x,
										y: y,
										curve: {
											type: 'cubic',
											x1: x1,
											y1: y1,
											x2: x2,
											y2: y2
										}
									});
							}
							i--;
					} else if (type === 'cubic') {
						points.splice(i, 1);
						_straightLines = CubicBezierApprox(p0.x, p0.y, x1, y1, x2, y2, p.x, p.y);
							for (var i2 = 0, len = _straightLines.length; i2 < len; i2++) {
								points.splice(i + i2, 0, _straightLines[i2]);
							}
					} else if (type === 'quadratic') {
						points.splice(i, 1);
						_straightLines = QuadraticBezierApprox(p0.x, p0.y, x1, y1, p.x, p.y);
							for (var i2 = 0, len = _straightLines.length; i2 < len; i2++) {
								points.splice(i + i2, 0, _straightLines[i2]);
							}
					}
			} else {
				i++;
			}
	}
	return points
}

var getY = function (points, samples, width, height) {
	samples = samples !== undefined ? samples : 300000;
		width = width !== undefined ? width : 100;
		height = height !== undefined ? height : 100;

		points = ApproximateCurves(points);
		var xSamples = [];
		var ySamples = [];
		var k = points.length - 1;
		var delta = 1 / samples;
		var t = 0;

		var firstPoint = points[0];

		xSamples.push(firstPoint.x / width);
		ySamples.push(1 - (firstPoint.y / height));

		for (var i = 0; i < samples; i++) {
			t += delta;

			var p = t * k;
				var f = Math.floor(p);
				var dt = p - f;
				var m = f + 1 > k ? k : f + 1;

				var p1 = points[f];
				var p2 = points[m];

				xSamples.push((p1.x + (p2.x - p1.x) * dt) / width);
				ySamples.push(1 - ((p1.y + (p2.y - p1.y) * dt) / height));
		}

		var lastPoint = points[points.length - 1];
		var lastX = lastPoint.x / width;
		var lastY = 1 - (lastPoint.y / height);

		xSamples.push(lastX);
		ySamples.push(lastY);

		var isEndAtOne = lastY === 1;

		return function (t) {
		for (var n = 0, len = xSamples.length; n < len; n++) {
			if (t === xSamples[n] || (xSamples[n] < t && xSamples[n + 1] > t)) {
				return ySamples[n]
			}
		}
		return t === 1 ? isEndAtOne ? t : lastY : isEndAtOne ? t : ySamples[(t * ySamples.length - 1) | 0]
	}
};

function OwnEasing(d, accuracy, width, height) {
	return getY(d, accuracy ? 100 * accuracy : 300, width, height)
}

// To-Do: Interpolation drawing support
OwnEasing.Value = function (d, accuracy, width, height) {
	var points = ApproximateCurves(d);
		return function (arr, t) {
		var l = arr.length - 1;
			var k = t * l;
			var i = Math.floor(k);
			var m = i + 1 > l ? l : i + 1;
			var py = (getPointAtElapsed(points, t).y / 100) - 0.5;
			return arr[i] + (arr[m] - arr[i]) * py
	}
};

return OwnEasing;

})));
